/*  dumper - hex, dec, oct dump
 *  Copyright (C) 2005 Dmitri A. Alenitchev
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 *  $Id$
 *
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <getopt.h>
#include <string.h>
#include "modes.h"
#include "version.h"

const char* program_name;
const char* output_filename = NULL;

void print_usage(FILE* stream, int exit_code);
void print_version();

int 
main(int argc, char* argv[])
{
  int fd;
  int next;
  int ascii = 0;
  int verbose = 0;
  FILE* out;

  const char* const short_opts = "avo:hV";
  const struct option long_opts[] = {
    { "ascii",   0, NULL, 'a' },
    { "verbose", 0, NULL, 'v' },
    { "output",  1, NULL, 'o' },
    { "help",    0, NULL, 'h' },
    { "version", 0, NULL, 'V' },
    { NULL,      0, NULL, 0   }
  };
  program_name = argv[0];

  while ((next = getopt_long(argc, argv, short_opts, long_opts, NULL)) != -1) {
    switch (next)
    {
    case 'a':
      ascii = 1;

    case 'v':
      verbose = 1;

    case 'o':
      output_filename = optarg;
      break;

    case 'h':
      print_usage(stdout, 0);

    case 'V':
      print_version();

    case '?':
      print_usage(stderr, 1);

    case -1:
      break;

    default:
      abort();
    }
  }

  while (optind != argc)
    {
      if (verbose)
	fprintf(stderr, "==> %s <==\n", argv[optind]);

      fd = open(argv[optind], O_RDONLY);
      if (fd == -1) /* Cann't open input file */
	{
	  fprintf(stderr, "%s: open(%s) failed: %s\n", 
		  program_name, argv[optind], strerror(errno));
	} 
      else          /* Input file was opened */
	{
	  if (output_filename) /* Write output in file */
	    {
	      out = fopen(output_filename, "a");
	      if (out == NULL) /* Cann't open output file */
		{ 
		  fprintf(stderr, "%s: fopen(%s) failed: %s\n", 
			  program_name, output_filename, strerror(errno));
		  exit(2);
		}
	    }
	  else                 /* Default, write in stdout */
	    out = stdout;

	  print_hex_dump(fd, out, ascii);
	  close(fd);
	}
      if (verbose)
	fprintf(stderr, "\n");

      ++optind;
    }

  return 0;
}

void 
print_usage(FILE* stream, int exit_code)
{
  fprintf(stream, "Usage: %s [options] [file]\n", program_name);
  fprintf(stream,
	  "   -a  --ascii                 Add ascii to dump.\n"
	  "   -v  --verbose               Verbose output.\n"
	  "   -o  --output filename       Write output to file.\n"
	  "   -V  --version               Display version.\n"
	  "   -h  --help                  Display this usage infofmation.\n");
  exit(exit_code);
}

void
print_version()
{
  printf("dumper version %s\nCopyright (C) 2005 Dmitri A. Alenitchev\n", 
	 version);
  exit(0);
}
